#!/usr/bin/env python

#       dizzy.py
#       
#       Copyright 2011 Daniel Mende <mail@c0decafe.de>
#

#       Redistribution and use in source and binary forms, with or without
#       modification, are permitted provided that the following conditions are
#       met:
#       
#       * Redistributions of source code must retain the above copyright
#         notice, this list of conditions and the following disclaimer.
#       * Redistributions in binary form must reproduce the above
#         copyright notice, this list of conditions and the following disclaimer
#         in the documentation and/or other materials provided with the
#         distribution.
#       * Neither the name of the  nor the names of its
#         contributors may be used to endorse or promote products derived from
#         this software without specific prior written permission.
#       
#       THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#       "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#       LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#       A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#       OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#       SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#       LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#       DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#       THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#       (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#       OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import math
from optparse import OptionParser
import os
import platform
import pprint
import random
import signal
import socket
import struct
import sys
import time

import dnet
import pcap

VERSION = "0.5"
PLATFORM = platform.system()
DEBUG = False

RANDOM_SEED="1l0v3D1zzYc4us31tsR4nd0m1sr3Pr0duc4bl3!"
random.seed(RANDOM_SEED)

class dizz_sessionException(Exception):
    pass

class dizz_session(object):
    def __init__(self, type, interface=None, pcap_filter=None, dest=None, dport=None, src='', sport=None, timeout=1, recv_buffer=4096, filename=None, auto_reopen=True):
        self.type = type
        self.timeout = timeout
        self.recv_buffer = recv_buffer
        if type == "eth":
            self.interface = interface
            self.pcap_filter = pcap_filter
            return
        elif type == "udp":
            self.sock = socket.SOCK_DGRAM
        elif type == "tcp":
            self.sock = socket.SOCK_STREAM
        elif type == "sctp":
            self.sock = socket.SOCK_SEQPACKET
        elif type == "stdout":
            return
        elif type == "file":
            if os.path.isfile(filename):
                self.filename = filename
            else:
                raise dizz_sessionException("unknown file: %s" % filename)
                return
        else:
            raise dizz_sessionException("unknown type: %s" % type)
        try:
            tmp = dnet.ip_aton(dest)
            self.af = socket.AF_INET
        except:
            try:
                tmp = dnet.ip6_aton(dest)
                self.af = socket.AF_INET6
            except:
                raise dizz_sessionException("unknown address family: %s" % dest)
        self.dest = dest
        self.dport = dport
        self.src = src
        self.sport = sport
        self.is_open = False
        self.auto_reopen = auto_reopen

    def open(self):
        try:
            if self.type == "eth":
                self.d = dnet.eth(self.interface)
                self.p = pcap.pcapObject()
                self.p.open_live(self.interface, 1600, self.timeout, 100)
                if self.pcap_filter:
                    self.p.setfilter(self.pcap_filter, 0, 0)
            elif self.type == "file":
                self.f = open(self.filename, 'w')
            elif self.type == "stdout":
                self.f = sys.stdout
            else:
                self.s = socket.socket(self.af, self.sock)
                self.s.settimeout(self.timeout)
                if self.sport:
                    self.s.bind((self.src, self.sport))
                if self.type == "tcp":
                    self.s.connect((self.dest, self.dport))
        except Exception, e:
            raise dizz_sessionException("cant open session: %s" % str(e))
        else:
            self.is_open = True
    
    def close(self):
        if self.type == "eth":
            self.d = None
            self.p = None
        elif self.type == "file":
            close(self.f)
        elif self.type == "stdout":
            pass
        else:
            self.s.close()
            self.s = None
        self.is_open = False

    def send(self, data):
        try:
            if self.type == "eth":
                self.d.send(data)
            elif self.type == "file":
                self.f.write(data)
            elif self.type == "stdout":
                self.f.write(data + "\n")
            elif self.type == "tcp":
                self.s.send(data)
            else:
                self.s.sendto(data, (self.dest, self.dport))
        except Exception, e:
            self.close()
            if self.auto_reopen:
                if DEBUG:
                    print "session got closed '%s', autoreopening..." % str(e)
                self.open()
            else:
                raise dizz_sessionException("error on sending '%s', connection closed." % str(e))
    
    def recv(self):
        if self.type == "eth":
            #raise dizz_sessionException("recv on eth is not yet implemented")
            return self.p.next()
        elif self.type == "file":
            return None
        elif self.type == "stdout":
            line = sys.stdin.readline()
            if line == ".\n":
                return None
            else:
                return line
        else:
            return self.s.recv(self.recv_buffer)

class dizz_library(object):
    def __init__(self):
        self.lib = {}
        self.load_strings("lib/std_string_lib.txt")

    def get_next(self, obj):
        libidx = obj["length"]
        cur = obj["cur"]
        if obj["_type"] == "list":
            libidx = obj["listname"]
        if not libidx in self.lib:
            self.gen_entries(libidx)
        if cur not in self.lib[libidx]:
            return None
        return self.lib[libidx][self.lib[libidx].index(cur) + 1]

    def gen_entries(self, length):
        bytelen = length / 8
        if length % 8 > 0:
            bytelen += 1
        if length >= 4:
            entr = []
            entr += [struct.pack("!Q", 0)[-bytelen:]]
            entr += [struct.pack("!Q", 1)[-bytelen:]]
            entr += [struct.pack("!Q", 2)[-bytelen:]]
            entr += [struct.pack("!Q", 3)[-bytelen:]]
            entr += [struct.pack("!Q", 4)[-bytelen:]]
            max = math.pow(2, length) - 1
            entr += [struct.pack("!Q", max - 4)[-bytelen:]]
            entr += [struct.pack("!Q", max - 3)[-bytelen:]]
            entr += [struct.pack("!Q", max - 2)[-bytelen:]]
            entr += [struct.pack("!Q", max - 1)[-bytelen:]]
            entr += [struct.pack("!Q", max)[-bytelen:]]
            entr += [None]
        elif length == 3:
            entr = ["\x00", "\x01", "\x02", "\x03", "\x04", "\x05", "\x06", "\x07", None]
        elif length == 2:
            entr = ["\x00", "\x01", "\x02", "\x03", None]
        elif length == 1:
            entr = ["\x00", "\x01", None]
        self.lib[length] = entr

    def load_strings(self, filename, listname=None):
        if listname in self.lib:
            return
        self.lib[listname] = [ "" , None ]
        with open(filename, 'r') as f:
            for l in f:
                self.lib[listname].insert(-1, l.rstrip('\n'))
    
class dizz_parseException(Exception):
    pass

class dizz_runtimeException(Exception):
    pass

class dizz(object):
    def __init__(self, library=None):
        self.objects = []
        self.obj_dict = {}
        self.functions = []
        self.cur_obj = None
        if not library:
            library = dizz_library()
        self.library = library
        self.filename = None
        
    def update_obj_dict(self):
	self.obj_dict = {}
        for i in self.objects:
            self.obj_dict[i["_name"]] = i

    def load(self, filename):
        self.filename = filename
        ns = {  "field"     :   self.basic_dizz,
                "list"      :   self.list_dizz,
                "rand"      :   self.rand_dizz,
                "link"      :   self.link_dizz,

                "none"      :   "none",
                "std"       :   "std",
                "full"      :   "full",
                
                "length"    :   self.basic_length
                }
        execfile(filename, ns)
        self.name = ns["name"]
        self.objects = ns["objects"]
        self.functions = ns["functions"]
        self.update_obj_dict()

    def save(self, filename):
        pp = pprint.PrettyPrinter(indent=4)
        with open(filename, 'w') as f:
            f.write("#%s %s autogenerated\n" % (self.__class__.__name__, VERSION))
            f.write("name = \"%s\"\n\n" % self.name)
            class ndict(dict):
                def __repr__(self):
                    if self["_type"] == "basic":
                        if self["length"] == None:
                            return "field('%s', None, '%s', '%s')" % (self["_name"], self["default"].encode("string_escape"), self["fuzz"])
                        else:
                            return "field('%s', %d, '%s', '%s')" % (self["_name"], self["length"], self["default"].encode("string_escape"), self["fuzz"])
                    elif self["_type"] == "list":
                        return "list('%s', '%s', '%s')" % (self["_name"], self["default"].encode("string_escape"), self["listname"])
                    elif self["_type"] == "rand":
                        return "rand('%s', %d)" % (self["_name"], self["length"])
                    elif self["_type"] == "link":
                        return "link('%s', '%s')" % (self["_name"], self["source"])
            tmpobj = []
            for i in self.objects:
                n = ndict(i)
                tmpobj += [ n ]
            f.write("objects = %s\n\n" % pp.pformat(tmpobj))
            class ndict(dict):
                def __repr__(self):
                    if self["func"] == "length":
                        return "length('%s', '%s', '%s')" % (self["dest"], self["start"], self["end"])
            tmpobj = []
            for i in self.functions:
                n = ndict(i)
                tmpobj += [ n ]
            f.write("functions = %s\n" % pp.pformat(tmpobj))

    def _alignmod(self, obj, mod):
        out = ""
        out += chr((ord(obj["cur"][0]) << 8 - mod) & 0xff)
        for j in xrange(1, obj["bytelen"]):
            out = out[:-1] + chr(ord(out[-1:]) | ord(obj["cur"][j]) >> mod)
            tmp = chr((ord(obj["cur"][j]) << 8 - mod) & 0xff)
            out += tmp
        return out

    def generate(self, offset=0, leading_data="\x00"):
        _DEBUG = False
        if offset == 0:
            out = ""
        else:
            out = leading_data
        for i in self.objects:
            if _DEBUG: print "offset: %d" % offset 
            if not i["length"]:
                bytelen = len(i["cur"])
                length = bytelen * 8
            else:
                length = i["length"]
                bytelen = i["bytelen"]
            modulo = length % 8
            if _DEBUG: print "modulo: %d" % modulo
            if offset != 0:
                if modulo != 0:
                    cur = self._alignmod(i, modulo)
                else:
                    cur = i["cur"]
                for j in xrange(bytelen):
                    out = out[:-1] + chr(ord(out[-1:]) | (ord(cur[j]) >> 8 - offset))
                    tmp = chr((ord(cur[j]) << offset) & 0xff)
                    if bytelen > 1:
                        if j == bytelen -1:
                            if abs(offset - modulo) == 0:
                                break
                        out += tmp
                offset = abs(offset - modulo)
            else:
                if modulo != 0:
                    out += self._alignmod(i, modulo)
                    offset = 8 - modulo
                else:
                    out += i["cur"]
        return out

    def _nullobj(self, obj):
        if obj["fuzz"] == "none":
            return
        out = ""
        if obj["bytelen"]:
            for i in xrange(obj["bytelen"]):
                out += "\x00"
        obj["cur"] = out

    def _nextobj(self):
        self.objects[self.cur_obj]["cur"] = self.objects[self.cur_obj]["default"]
        self.cur_obj += 1
        if self.cur_obj < len(self.objects):
            self._nullobj(self.objects[self.cur_obj])
        
    def mutate(self):
        _DEBUG = False
        done = False
        while not done:
            if self.cur_obj == None:
                self.cur_obj = 0
                while self.objects[self.cur_obj]["fuzz"] == "none":
                    self.cur_obj += 1
                    if self.cur_obj >= len(self.objects):
                        self.cur_obj = None
                        return False
                self._nullobj(self.objects[self.cur_obj])
                break
            if self.cur_obj >= len(self.objects):
                if _DEBUG: print "EOMutate: %d %d" % (self.cur_obj, len(self.objects))
                self.cur_obj = None
                return False
            else:
                if self.objects[self.cur_obj]["fuzz"] == "none":
                    self._nextobj()
                elif self.objects[self.cur_obj]["fuzz"] == "full":
                    if _DEBUG:
                        print "cur: %s, int(cur): %d, max: %d" % (self.objects[self.cur_obj]["cur"].encode("hex"), long(self.objects[self.cur_obj]["cur"].encode("hex"), 16), math.pow(2, self.objects[self.cur_obj]["length"]) - 1)
                    if long(self.objects[self.cur_obj]["cur"].encode("hex"), 16) >= math.pow(2, self.objects[self.cur_obj]["length"]) - 1:
                        self._nextobj()
                        done = True
                    else:
                        self.objects[self.cur_obj]["cur"] = struct.pack("!Q", long(self.objects[self.cur_obj]["cur"].encode("hex"), 16) + 1)[-self.objects[self.cur_obj]["bytelen"]:]
                        done = True
                elif self.objects[self.cur_obj]["fuzz"] == "std":
                    next = self.library.get_next(self.objects[self.cur_obj])
                    if _DEBUG: print "len: %s cur: %s next: %s" % (self.objects[self.cur_obj]["length"], self.objects[self.cur_obj]["cur"], next)
                    if not next:
                        self._nextobj()
                        done = True
                    else:
                        self.objects[self.cur_obj]["cur"] = next
                        done = True
                else:
                    raise dizz_runtimeException("unknown fuzzing type: %s" % self.objects[self.cur_obj]["fuzz"])
        if self.cur_obj >= len(self.objects):
            if _DEBUG: print "EOMutate: %d %d" % (self.cur_obj, len(self.objects))
            self.cur_obj = None
            return False
        return True

    def operate(self):
        _DEBUG = False
        for i in self.functions:
            if i["func"] == "length":
                len_index = self.objects.index(self.obj_dict[i["dest"]])
                start_index = self.objects.index(self.obj_dict[i["start"]])
                end_index = self.objects.index(self.obj_dict[i["end"]])
                length = 0
                for j in xrange(start_index, end_index + 1):
                    if self.objects[j]["length"]:
                        length += self.objects[j]["length"]
                    else:
                        length += len(self.objects[j]["cur"]) * 8
                length = length / 8
                self.objects[len_index]["cur"] = struct.pack("!L", length)[-self.objects[len_index]["bytelen"]:]
                if _DEBUG:
                    print "LENGTH: dest: %s, start: %s, end: %s, len: %d" % (i["dest"], i["start"], i["end"], length)
        for i in self.objects:
            if i["_type"] == "rand":
                new_rand = ""
                for j in xrange(i["bytelen"]):
                    new_rand += chr(random.randint(0x00, 0xff))
                i["cur"] = new_rand
            elif i["_type"] == "link":
                i["cur"] = self.obj_dict[i["source"]]["cur"]
    
    def basic_dizz(self, name, length, default, fuzz):
        if not name:
            raise dizz_parseException("no name defined!")
        if length:
            bytelen = length / 8
            if length % 8 != 0:
                bytelen += 1
            if len(default) != bytelen:
                raise dizz_parseException("length of default value doesnt match length attribute: %s" % name)
        else:
            bytelen = None
        if not length and fuzz == "full":
            raise dizz_parseException("cannot make dizz without length full fuzzable: %s" % name)
        if length > sys.maxint and fuzz == "full":
            raise dizz_parseException("cannot make dizz with length '%d' full fuzzable, this would take ages: %s" % (length, name))
        if name in self.obj_dict:
            raise dizz_parseException("dizz with name '%s' already exists!" % name)
        obj  = {    "_type"     :   "basic",
                    "_name"     :   name,
                    "length"    :   length,
                    "default"   :   default,
                    "fuzz"      :   fuzz,

                    "bytelen"   :   bytelen,
                    "cur"       :   default
                    }
        self.obj_dict[name] = obj
        return obj

    def list_dizz(self, name, default, listname):
        obj = self.basic_dizz(name, None, default, "std")
        obj["_type"] = "list"
        obj["listname"] = listname
        self.library.load_strings(listname, listname)
        return obj
        
    def link_dizz(self, name, source):
        if source not in self.obj_dict:
            raise dizz_parseException("cannot find source dizz %s" % source)
        obj = self.basic_dizz(name, self.obj_dict[source]["length"], self.obj_dict[source]["default"], "none")
        obj["_type"] = "link"
        obj["source"] = source
        return obj
    
    def rand_dizz(self, name, length):
        if not length:
            raise dizz_parseException("cannot create random dizz without length: %s" % name)
        bytelen = length / 8
        if length % 8 != 0:
            bytelen += 1
        obj = self.basic_dizz(name, length, "\x00" * bytelen, "none")
        obj["_type"] = "rand"
        return obj

    def basic_length(self, dest, start, end):
        if dest not in self.obj_dict:
            raise dizz_parseException("destination field '%s' unknown!" % dest)
        if start not in self.obj_dict:
            raise dizz_parseException("start field '%s' unknown!" % start)
        if end not in self.obj_dict:
            raise dizz_parseException("end field '%s' unknown!" % end)
        return {    "func"      :   "length",
                    "dest"      :   dest,
                    "start"     :   start,
                    "end"       :   end
                    }

class interact_parseException(Exception):
    pass

class interaction(object):
    def __init__(self, library=None):
        self.objects = []
        self.functions = []
        if not library:
            library = dizz_library()
        self.library = library
        self.cur_obj = None
        self.gen_obj = None

    def load(self, filename):
        self.filename = filename
        ns = {  "dizz"  :   self.dizz_obj,
                "copy"  :   self.basic_copy,
                "adv_copy"  :   self.adv_copy,
                }
        execfile(filename, ns)
        self.name = ns["name"]
        self.objects = ns["objects"]
        self.functions = ns["functions"]

    def save(self, filename):        
        pp = pprint.PrettyPrinter(indent=4)
        with open(filename, 'w') as f:
            f.write("#%s %s autogenerated\n" % (self.__class__.__name__, VERSION))
            f.write("name = \"%s\"\n\n" % self.name)
            class ndict(dict):
                def __repr__(self):
                    return "dizz('%s', '%s')" % (self["name"], self["dizz"].filename)
            tmpobj = []
            for i in self.objects:
                n = ndict(i)
                tmpobj += [ n ]
            f.write("objects = %s\n\n" % pp.pformat(tmpobj))
            class ndict(dict):
                def __repr__(self):
                    if self["func"] == "copy":
                        return "copy(%d, '%s', %d, %d)" % (self["step"], self["dest"], self["start"], self["end"])
                    elif self["func"] == "adv_copy":
                        return "adv_copy(%d, '%s')" % (self["step"], self["callback"].func_name)
            tmpobj = []
            for i in self.functions:
                n = ndict(i)
                tmpobj += [ n ]
            f.write("funktions = %s\n" % pp.pformat(tmpobj))

    def dizz_obj(self, name, filename):
        if not name:
            raise interact_parseException("no name defined!")

        obj = {}
        obj["name"] = name
        obj["dizz"] = dizz(self.library)
        obj["dizz"].load(filename)
        obj["dizz"].operate()
        return obj

    def _next_obj(self):
        if self.cur_obj + 1 < len(self.objects):
            self.cur_obj += 1
            return True
        return False

    def generate(self):
        _DEBUG = False
        ret = ""
        recv = False
        if self.cur_obj == None:
            self.cur_obj = 0
        if self.gen_obj == None:
            self.gen_obj = 0

        if _DEBUG:
            print "cur: %d\tgen: %d" % (self.cur_obj, self.gen_obj)

        if self.gen_obj < self.cur_obj:
            self.objects[self.gen_obj]["dizz"].operate()
            ret = self.objects[self.gen_obj]["dizz"].generate()
            self.gen_obj += 1
        else:
            if not self.objects[self.cur_obj]["dizz"].mutate():                
                self.objects[self.cur_obj]["dizz"].operate()
                ret = self.objects[self.cur_obj]["dizz"].generate()
                if not self._next_obj():
                    if _DEBUG:
                        print "EOInteract"
                    return None
            else:
                self.gen_obj = 0
                self.objects[self.cur_obj]["dizz"].operate()
                ret = self.objects[self.cur_obj]["dizz"].generate()
        return ret

    def operate(self, input=None):
        _DEBUG = False
        if _DEBUG:
            print "in: %s" % input.encode("hex")
            print "cur: %d" % self.gen_obj
        if input and input != "":
            for i in self.functions:
                if i["step"] == self.gen_obj:
                    if i["func"] == "copy":
                            self.objects[self.gen_obj]["dizz"].obj_dict[i["dest"]]["cur"] = input[i["start"]:i["end"]]
                    elif i["func"] == "adv_copy":
                             i["callback"](self.objects[self.gen_obj], input)
        
    def basic_copy(self, step, dest, start, end):
        obj = { "func"  :   "copy",
                "step"  :   step,
                "dest"  :   dest,
                "start" :   start,
                "end"   :   end,
                "callback"  :   None,
                }
        return obj
    
    def adv_copy(self, step, call):
        obj = { "func"  :   "adv_copy",
                "step"  :   step,
                "callback"  :   call,
                }
        return obj
        
def chr_to_bin(c):
    out = ""
    for i in xrange(0,8):
        if i == 4:
            out += " "
        out += "%d" % (((ord(c) << i) & 0x80) >> 7)
    return out

def str_to_bin(s):
    out = ""
    c = 1
    for i in s:
        out += chr_to_bin(i)
        if c % 8 == 0:
            out += "\n"
        else:
            out += "  "
        c += 1
    return out[:-2]

if __name__ == '__main__':
    parser = OptionParser(usage="usage: %s [options] {dizzfile | ackfile}" % os.path.basename(sys.argv[0]), version=VERSION)
    parser.add_option("-v", help="Be verbose", dest="verbose", action="store_true", default=False)
    parser.add_option("-t", help="Dont mutate, just send first package", dest="test", action="store_true", default=False)
    parser.add_option("-o", help="Output type {eth, udp, tcp, sctp, file, stdout}", choices=["eth", "udp", "tcp", "sctp", "file", "stdout"], dest="out_type", default="stdout")
    parser.add_option("-d", type="string", help="Output destination (interface for eth, dst-ip for udp/tcp/sctp, filename for file)", dest="out_dest", default=None)
    parser.add_option("-e", type="string", help="Output extra args (pcap-filter for eth, src:dst port for udp/tcp/sctp)", dest="out_extra", default=None)
    parser.add_option("-w", type="float", help="Time to wait between mutations (default 1)", metavar="SEC", dest="wait_send", default=1)
    parser.add_option("-W", type="float", help="Time to wait on receive (default 1)", metavar="SEC", dest="wait_recv", default=1)
    (options, args) = parser.parse_args()
    if len(args) != 1:
        parser.error("incorrect number of arguments")

    if options.verbose:
        DEBUG=True

    if options.out_type == "eth":
        if os.geteuid() != 0:
            print "You must be root to send on eth."
            sys.exit(1)
        try:
            s = dizz_session(options.out_type, interface=options.out_dest, pcap_filter=options.out_extra, timeout=options.wait_recv)
        except Exception, e:
            parser.error("invalid arguments %s:%s : %s" % (options.out_dest, options.out_extra, e))
            sys.exit(1)
    elif options.out_type == "file":
        try:
            s = dizz_session(options.out_type, filename=options.out_dest)
        except Exception, e:
            parser.error("invalid arguments %s : %s" % (options.out_dest, e))
            sys.exit(1)
    elif options.out_type == "stdout":
        s = dizz_session(options.out_type)
    else:
        try:
            ports = options.out_extra.split(":")
            s = dizz_session(options.out_type, dest=options.out_dest, sport=int(ports[0]), dport=int(ports[1]), timeout=options.wait_recv)
        except Exception, e:
            parser.error("invalid arguments %s:%s : %s" % (options.out_dest, options.out_extra, e))
            sys.exit(1)
            
    l = dizz_library()
    dot = args[0].rfind(".")
    if args[0][dot:] == ".act":
        i = interaction(l)
        try:
            i.load(args[0])
        except Exception, e:
            parser.error("invalid argument: %s: %s" % (args[0], e))
            print e.stacktrace()
            sys.exit(1)
        s.open()
        o = i.generate()
        d = None
        run = True
        while run:
            if DEBUG:
                print "w: " + o.encode("hex")
                #print str_to_bin(o)
            try:
                s.send(o)
            except Exception, e:
                print "Cant write output: %s" % e
                sys.exit(1)
            d = ""
            try:
                reading = True
                while reading:
                    r = s.recv()
                    if not r:
                        reading = False
                        if DEBUG:
                            print "Read end on NONE"
                    else:
                        d += r
                        if DEBUG:
                            print "r: " + d.encode("hex")
                            #print str_to_bin(d)                        
            except socket.timeout:
                if DEBUG:
                    print "Read end on timeout"
                pass
            except Exception, e:
                print "Cant read input: %s" % e
                print e.stacktrace()
                sys.exit(1)
            i.operate(d)
            o = i.generate()
            if o == None:
                run = False
            if options.test:
                break
            time.sleep(options.wait_send)
    else:
        d = dizz(l)
        try:
            d.load(args[0])
        except Exception, e:
            parser.error("invalid argument: %s: %s" % (args[0], e))
            print e.stacktrace()
            sys.exit(1)
        s.open()
        run = True
        while run:
            d.operate()
            o = d.generate()
            if DEBUG:
                print ""
                print o.encode("hex")
                #print str_to_bin(o)
            try:
                s.send(o)
            except Exception, e:
                print "Cant write output: %s" % e            
            if options.test:
                break
            run = d.mutate()
            time.sleep(options.wait_send)
    s.close()
    #~ if DEBUG:
        #~ pp = pprint.PrettyPrinter()
        #~ pp.pprint(l.lib)

    sys.exit(0)
